@@ -4,6 +4,7 @@ import numpy as np
 
 import warnings
 import pickle
+import ast
 
 from .core import Dataset
 
@@ -46,20 +47,49 @@ def read_ekpds(filename):
 		out = f.read()
 
 	_location1 = out.find(b'########') #up to here is preamble, after is readfileby
-	pointercolumn = pickle.loads(out[:_location1])
+	preamble = pickle.loads(out[:_location1])
+	preamble_spl = preamble.split('|')
+	preamble_dict = {}
+
+	y = None
+	for x in preamble_spl:
+		if 'pointercolumn' in x:
+			key, value = x.split(':')
+			preamble_dict.update({key:value})
+		else:
+			if type(y) == type(None):
+				y = x
+			else:
+				raise ValueError('error in load. expected 2 items in preamble, recieved at least 3. This likely originated from a change in .to_ekpds without appropriate changes to reading ekpds.')
+
+	assert 'path:' in y, '"path:" not in preamble, or incorrectly passed.'
+	path = ast.literal_eval(y.split('path:')[-1])
+	preamble_dict.update({'path':path})
+
 	readfileby = pickle.loads(out[_location1+8:])
 	_location2 = out.find(b'##|##|##|##') #after this is dset
-	dset = pickle.loads(out[_location2+11:])
+	meta_data = pickle.loads(out[_location2+11:])
 	
-	return Dataset(dset.path, pd.DataFrame(dset), readfileby)
+	return Dataset(preamble_dict['path'], pd.DataFrame(meta_data), readfileby, pointercolumn=preamble_dict['pointercolumn'])
 
 def _build_df(path, meta_data):
+
+	# I have noticed I'm getting an attribute error when the pickle file is created by a different version of pandas... TODO find the origin of this error, for now, updating to store meta_data as a csv as well in control.experiment
 	if type(meta_data) == type(None):
 		try:
 			return pd.read_pickle(path + 'meta_data')
 		except FileNotFoundError:
-			print('meta_data does not exist in path {} you may want to create it with generate_meta_data()'.format(path))
-			return pd.DataFrame()
+			try:
+				return pd.read_csv(path + 'meta_data.csv')
+			except FileNotFoundError:
+				print('No file named "meta_data" exists in path "{}", you may want to create one or both of these with `.generate_meta_data()`'.format(path, path))
+				return pd.DataFrame()
+		except AttributeError:
+			try:
+				return pd.read_csv(path + 'meta_data.csv')
+			except FileNotFoundError:
+				print('Pickle file "meta_data" exists in path "{}", but failed to load (likely do to incompatible pandas versions). Backup, "meta_data.csv" does not exist in "{}", you may want to create one or both of these with `.generate_meta_data()`'.format(path, path))
+				return pd.DataFrame()
 	else:
 		return meta_data
 
